local xml2lua = require 'xml2lua'
local handler = require 'xmlhandler.tree'
local ftcsv = require 'ftcsv'
local parser = xml2lua.parser(handler)
local map = {}
local tileset = {}

function map:load(...)
  map:loadTMX(love.filesystem.getWorkingDirectory() .. "/map/map.tmx")
end

function tileset:load(...)
  tileset:loadTSX(love.filesystem.getWorkingDirectory() .. "/map/tilemap_packed.tsx")
  tileset:loadTile()
end

function map:draw()
  for _, layer in ipairs(map) do
    for i = 1, map.height, 1 do
      for k = 1, map.width, 1 do
        if layer[i][k] ~= "0" then
          love.graphics.draw(
            tileset.image,
            tileset[tonumber(layer[i][k])],
            tileset.tilewidth * (k - 1),
            tileset.tileheight * (i - 1)
          )
        end
      end
    end
  end
end

function openFile(filename)
  local f = io.open(filename, 'r')
  if f then
    return f
  else
    error("no file exists", 2)
  end
end

function map:loadTMX(filename)
  local f = openFile(filename)
  local tmx = f:read('a')
  parser:parse(tmx)
  local attr = handler.root.map._attr
  map.width = tonumber(attr.width)
  map.height = tonumber(attr.height)
  for _, value in pairs(handler.root.map.layer) do
    map[#map + 1] = ftcsv.parse(value.data[1] .. ',', ",", { headers = false, loadFromString = true })
  end
end

function tileset:loadTSX(filename)
  local f = openFile(filename)
  local tsx = f:read('a')
  parser:parse(tsx)
  local attr = handler.root.tileset._attr
  local image = handler.root.tileset.image._attr
  tileset.tilewidth = tonumber(attr.tilewidth)
  tileset.tileheight = tonumber(attr.tileheight)
  tileset.tilecount = tonumber(attr.tilecount)
  tileset.columns = tonumber(attr.columns)
  tileset.rows = tileset.tilecount / tileset.columns
  tileset.source = image.source
  tileset.width = tonumber(image.width)
  tileset.width = tonumber(image.height)
end

function tileset:loadTile()
  self.source = string.gsub(self.source, "%.%.", "")
  self.image = love.graphics.newImage(self.source)
  for i = 0, tileset.rows - 1, 1 do
    for j = 0, tileset.columns - 1, 1 do
      tileset[#tileset + 1] = love.graphics.newQuad(
        j * tileset.tilewidth, i * tileset.tileheight,
        tileset.tilewidth, tileset.tileheight, self.image
      )
    end
  end
end

return {
  map = map,
  tileset = tileset
}
